@@ -1,31 +1,32 @@
 # This file is part of Supysonic.
 # Supysonic is a Python implementation of the Subsonic server API.
 #
-# Copyright (C) 2013-2020 Alban 'spl0k' Féron
+# Copyright (C) 2013-2023 Alban 'spl0k' Féron
 #
 # Distributed under terms of the GNU AGPLv3 license.
 
-API_VERSION = "1.10.2"
+API_VERSION = "1.12.0"
 
 import binascii
+import logging
 import uuid
 from flask import request
 from flask import Blueprint
-from pony.orm import ObjectNotFound, TransactionIntegrityError
-from pony.orm import commit
+from peewee import IntegrityError
 
 from ..db import ClientPrefs, Folder
 from ..managers.user import UserManager
 
-from .exceptions import GenericError, Unauthorized
+from .exceptions import GenericError, Unauthorized, NotFound
 from .formatters import JSONFormatter, JSONPFormatter, XMLFormatter
 
 api = Blueprint("api", __name__)
+logger = logging.getLogger(__name__)
 
 
 def api_routing(endpoint):
     def decorator(func):
-        viewendpoint = "{}.view".format(endpoint)
+        viewendpoint = f"{endpoint}.view"
         api.add_url_rule(endpoint, view_func=func, methods=["GET", "POST"])
         api.add_url_rule(viewendpoint, view_func=func, methods=["GET", "POST"])
         return func
@@ -58,12 +59,15 @@ def decode_password(password):
 @api.before_request
 def authorize():
     if request.authorization:
-        user = UserManager.try_auth(
-            request.authorization.username, request.authorization.password
-        )
+        username = request.authorization.username
+        user = UserManager.try_auth(username, request.authorization.password)
         if user is not None:
             request.user = user
             return
+
+        logger.error(
+            "Failed login attempt for user %s (IP: %s)", username, request.remote_addr
+        )
         raise Unauthorized()
 
     username = request.values["u"]
@@ -72,6 +76,9 @@ def authorize():
 
     user = UserManager.try_auth(username, password)
     if user is None:
+        logger.error(
+            "Failed login attempt for user %s (IP: %s)", username, request.remote_addr
+        )
         raise Unauthorized()
 
     request.user = user
@@ -82,15 +89,12 @@ def get_client_prefs():
     client = request.values["c"]
     try:
         request.client = ClientPrefs[request.user, client]
-    except ObjectNotFound:
+    except ClientPrefs.DoesNotExist:
         try:
-            request.client = ClientPrefs(user=request.user, client_name=client)
-            commit()
-        except TransactionIntegrityError:
+            request.client = ClientPrefs.create(user=request.user, client_name=client)
+        except IntegrityError:
             # We might have hit a race condition here, another request already created
             # the ClientPrefs. Issue #220
-            # Reload the user or Pony will complain about different transactions
-            request.user = UserManager.get(request.user.id)
             request.client = ClientPrefs[request.user, client]
 
 
@@ -100,8 +104,7 @@ def get_entity(cls, param="id"):
         eid = int(eid)
     else:
         eid = uuid.UUID(eid)
-    entity = cls[eid]
-    return entity
+    return cls[eid]
 
 
 def get_entity_id(cls, eid):
@@ -111,12 +114,27 @@ def get_entity_id(cls, eid):
             raise GenericError("Invalid ID")
         try:
             return int(eid)
-        except ValueError:
-            raise GenericError("Invalid ID")
+        except ValueError as e:
+            raise GenericError("Invalid ID") from e
     try:
         return uuid.UUID(eid)
-    except (AttributeError, ValueError):
-        raise GenericError("Invalid ID")
+    except (AttributeError, ValueError) as e:
+        raise GenericError("Invalid ID") from e
+
+
+def get_root_folder(id):
+    if id is None:
+        return None
+
+    try:
+        fid = int(id)
+    except ValueError as e:
+        raise ValueError("Invalid folder ID") from e
+
+    try:
+        return Folder.get(id=fid, root=True)
+    except Folder.DoesNotExist as e:
+        raise NotFound("Folder") from e
 
 
 from .errors import *
