@@ -1,7 +1,7 @@
 # This file is part of Supysonic.
 # Supysonic is a Python implementation of the Subsonic server API.
 #
-# Copyright (C) 2013-2020 Alban 'spl0k' Féron
+# Copyright (C) 2013-2022 Alban 'spl0k' Féron
 #               2018-2019 Carey 'pR0Ps' Metcalfe
 #
 # Distributed under terms of the GNU AGPLv3 license.
@@ -20,12 +20,11 @@ import zlib
 from flask import request, Response, send_file
 from flask import current_app
 from PIL import Image
-from pony.orm import ObjectNotFound
 from xml.etree import ElementTree
 from zipstream import ZipStream
 
 from ..cache import CacheMiss
-from ..db import Track, Album, Folder, now
+from ..db import Track, Album, Artist, Folder, now
 from ..covers import EXTENSIONS
 
 from . import get_entity, get_entity_id, api_routing
@@ -116,14 +115,14 @@ def stream_media():
     if dst_suffix != src_suffix or dst_bitrate != res.bitrate:
         # Requires transcoding
         cache = current_app.transcode_cache
-        cache_key = "{}-{}.{}".format(res.id, dst_bitrate, dst_suffix)
+        cache_key = f"{res.id}-{dst_bitrate}.{dst_suffix}"
 
         try:
             response = send_file(
                 cache.get(cache_key), mimetype=dst_mimetype, conditional=True
             )
         except CacheMiss:
-            transcoder = config.get("transcoder_{}_{}".format(src_suffix, dst_suffix))
+            transcoder = config.get(f"transcoder_{src_suffix}_{dst_suffix}")
             decoder = config.get("decoder_" + src_suffix) or config.get("decoder")
             encoder = config.get("encoder_" + dst_suffix) or config.get("encoder")
             if not transcoder and (not decoder or not encoder):
@@ -135,10 +134,10 @@ def stream_media():
                     logger.info(message)
                     raise GenericError(message)
 
-            transcoder, decoder, encoder = [
+            transcoder, decoder, encoder = (
                 prepare_transcoding_cmdline(x, res, src_suffix, dst_suffix, dst_bitrate)
                 for x in (transcoder, decoder, encoder)
-            ]
+            )
             try:
                 if transcoder:
                     dec_proc = None
@@ -211,9 +210,12 @@ def stream_media():
 
     res.play_count = res.play_count + 1
     res.last_play = now()
+    res.save()
+
     user = request.user
     user.last_play = res
     user.last_play_date = now()
+    user.save()
 
     return response
 
@@ -238,16 +240,16 @@ def download_media():
         try:
             rv = Track[uid]
             return send_file(rv.path, mimetype=rv.mimetype, conditional=True)
-        except ObjectNotFound:
+        except Track.DoesNotExist:
             try:  # Album -> stream zipped tracks
                 rv = Album[uid]
-            except ObjectNotFound:
-                raise NotFound("Track or Album")
+            except Album.DoesNotExist as e:
+                raise NotFound("Track or Album") from e
     else:
         try:  # Folder -> stream zipped tracks, non recursive
             rv = Folder[fid]
-        except ObjectNotFound:
-            raise NotFound("Folder")
+        except Folder.DoesNotExist as e:
+            raise NotFound("Folder") from e
 
     # Stream a zip of multiple files to the client
     z = ZipStream(sized=True)
@@ -278,22 +280,21 @@ def download_media():
         raise GenericError("Nothing to download")
 
     resp = Response(z, mimetype="application/zip")
-    resp.headers["Content-Disposition"] = "attachment; filename={}.zip".format(rv.name)
+    resp.headers["Content-Disposition"] = f"attachment; filename={rv.name}.zip"
     resp.headers["Content-Length"] = len(z)
     return resp
 
 
-def _cover_from_track(tid):
+def _cover_from_track(obj):
     """Extract and return a path to a track's cover art
 
     Returns None if no cover art is available.
     """
     cache = current_app.cache
-    cache_key = "{}-cover".format(tid)
+    cache_key = f"{obj.id}-cover"
     try:
         return cache.get(cache_key)
     except CacheMiss:
-        obj = Track[tid]
         try:
             return cache.set(cache_key, mediafile.MediaFile(obj.path).art)
         except mediafile.UnreadableFileError:
@@ -312,14 +313,16 @@ def _cover_from_collection(obj, extract=True):
         cover_path = os.path.join(obj.path, obj.cover_art)
 
     elif isinstance(obj, Album):
-        track_with_folder_cover = obj.tracks.select(
-            lambda t: t.folder.cover_art is not None
-        ).first()
+        track_with_folder_cover = (
+            obj.tracks.join(Folder, on=Track.folder)
+            .where(Folder.cover_art.is_null(False))
+            .first()
+        )
         if track_with_folder_cover is not None:
             cover_path = _cover_from_collection(track_with_folder_cover.folder)
 
         if not cover_path and extract:
-            track_with_embedded = obj.tracks.select(lambda t: t.has_art).first()
+            track_with_embedded = obj.tracks.where(Track.has_art).first()
             if track_with_embedded is not None:
                 cover_path = _cover_from_track(track_with_embedded.id)
 
@@ -328,11 +331,7 @@ def _cover_from_collection(obj, extract=True):
     return cover_path
 
 
-@api_routing("/getCoverArt")
-def cover_art():
-    cache = current_app.cache
-
-    eid = request.values["id"]
+def _get_cover_path(eid):
     try:
         fid = get_entity_id(Folder, eid)
     except GenericError:
@@ -345,15 +344,31 @@ def cover_art():
     if not fid and not uid:
         raise GenericError("Invalid ID")
 
-    cover_path = None
-    if fid and Folder.exists(id=eid):
-        cover_path = _cover_from_collection(get_entity(Folder))
-    elif uid and Track.exists(id=eid):
-        cover_path = _cover_from_track(eid)
-    elif uid and Album.exists(id=uid):
-        cover_path = _cover_from_collection(get_entity(Album))
-    else:
-        raise NotFound("Entity")
+    if fid:
+        try:
+            return _cover_from_collection(Folder[fid])
+        except Folder.DoesNotExist:
+            pass
+    elif uid:
+        try:
+            return _cover_from_track(Track[uid])
+        except Track.DoesNotExist:
+            pass
+
+        try:
+            return _cover_from_collection(Album[uid])
+        except Album.DoesNotExist:
+            pass
+
+    raise NotFound("Entity")
+
+
+@api_routing("/getCoverArt")
+def cover_art():
+    cache = current_app.cache
+
+    eid = request.values["id"]
+    cover_path = _get_cover_path(eid)
 
     if not cover_path:
         raise NotFound("Cover art")
@@ -366,21 +381,21 @@ def cover_art():
         # extension for Flask to derive the mimetype from - derive it from the
         # contents instead.
         mimetype = None
-        if uid and os.path.splitext(cover_path)[1].lower() not in EXTENSIONS:
+        if os.path.splitext(cover_path)[1].lower() not in EXTENSIONS:
             with Image.open(cover_path) as im:
-                mimetype = "image/{}".format(im.format.lower())
+                mimetype = f"image/{im.format.lower()}"
         return send_file(cover_path, mimetype=mimetype)
 
     with Image.open(cover_path) as im:
-        mimetype = "image/{}".format(im.format.lower())
+        mimetype = f"image/{im.format.lower()}"
         if size > im.width and size > im.height:
             return send_file(cover_path, mimetype=mimetype)
 
-        cache_key = "{}-cover-{}".format(eid, size)
+        cache_key = f"{eid}-cover-{size}"
         try:
             return send_file(cache.get(cache_key), mimetype=mimetype)
         except CacheMiss:
-            im.thumbnail([size, size], Image.ANTIALIAS)
+            im.thumbnail([size, size], Image.Resampling.LANCZOS)
             with cache.set_fileobj(cache_key) as fp:
                 im.save(fp, im.format)
             return send_file(cache.get(cache_key), mimetype=mimetype)
@@ -398,7 +413,11 @@ def lyrics():
     artist = request.values["artist"]
     title = request.values["title"]
 
-    query = Track.select(lambda t: title in t.title and artist in t.artist.name)
+    query = (
+        Track.select()
+        .join(Artist)
+        .where(Track.title.contains(title), Artist.name.contains(artist))
+    )
     for track in query:
         # Read from track metadata
         lyrics = mediafile.MediaFile(track.path).lyrics
@@ -424,11 +443,14 @@ def lyrics():
 
             return lyrics_response_for_track(track, lyrics)
 
+    if not current_app.config["WEBAPP"]["online_lyrics"]:
+        return request.formatter("lyrics", {})
+
     # Create a stable, unique, filesystem-compatible identifier for the artist+title
     unique = hashlib.md5(
         json.dumps([x.lower() for x in (artist, title)]).encode("utf-8")
     ).hexdigest()
-    cache_key = "lyrics-{}".format(unique)
+    cache_key = f"lyrics-{unique}"
 
     lyrics = {}
     try:
