@@ -1,7 +1,7 @@
 # This file is part of Supysonic.
 # Supysonic is a Python implementation of the Subsonic server API.
 #
-# Copyright (C) 2019 Alban 'spl0k' Féron
+# Copyright (C) 2019-2022 Alban 'spl0k' Féron
 #
 # Distributed under terms of the GNU AGPLv3 license.
 
@@ -9,8 +9,6 @@ import shutil
 import tempfile
 import unittest
 
-from pony.orm import db_session
-
 from supysonic.db import init_database, release_database
 from supysonic.managers.folder import FolderManager
 from supysonic.scanner import Scanner
@@ -20,14 +18,12 @@ class Issue139TestCase(unittest.TestCase):
     def setUp(self):
         self.__dir = tempfile.mkdtemp()
         init_database("sqlite:")
-        with db_session:
-            FolderManager.add("folder", self.__dir)
+        FolderManager.add("folder", self.__dir)
 
     def tearDown(self):
         release_database()
         shutil.rmtree(self.__dir)
 
-    @db_session
     def do_scan(self):
         scanner = Scanner()
         scanner.queue_folder("folder")
